@@ -1,7 +1,7 @@
 /*
  * lftp and utils
  *
- * Copyright (c) 1996-2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2001 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileAccess.h,v 1.56 2000/11/27 16:43:08 lav Exp $ */
+/* $Id: FileAccess.h,v 1.73 2002/01/21 16:25:14 lav Exp $ */
 
 #ifndef FILEACCESS_H
 #define FILEACCESS_H
 
 #include "SMTask.h"
-#include <stdio.h>
+#include "trio.h"
 #include <time.h>
 #include <sys/types.h>
 
@@ -91,8 +91,6 @@ protected:
    char  *user;
    char  *pass;
    bool	 pass_open;
-   char	 *group;
-   char	 *gpass;
 
    char	 *home;
    const char *default_cwd;
@@ -136,6 +134,12 @@ protected:
 	 xfree(real_cwd);
 	 real_cwd=xstrdup(c);
       }
+   void set_home(const char *h)
+      {
+	 xfree(home);
+	 home=xstrdup(h);
+	 ExpandTildeInCWD();
+      }
 
    char *url;
 
@@ -145,12 +149,17 @@ protected:
    char *closure;
    const char *res_prefix;
    ResValue Query(const char *name,const char *closure=0);
+   bool QueryBool(const char *name,const char *closure=0)
+      {
+	 return Query(name,closure).to_bool();
+      }
 
    int chmod_mode;
    bool ascii;
    bool norest_manual;
 
    int	priority;   // higher priority can take over other session.
+   int	last_priority;
 
    bool Error() { return error_code!=OK; }
    void ClearError();
@@ -183,15 +192,15 @@ public:
    const char  *GetPort() { return portname; }
    const char  *GetConnectURL(int flags=0);
    const char  *GetFileURL(const char *file,int flags=0);
-   enum { NO_PATH=1,WITH_PASSWORD=2,NO_PASSWORD=4 };
+   enum { NO_PATH=1,WITH_PASSWORD=2,NO_PASSWORD=4,NO_USER=8 };
 
    virtual void Connect(const char *h,const char *p);
    virtual void ConnectVerify();
-
    virtual void AnonymousLogin();
-
    virtual void Login(const char *u,const char *p);
-   virtual void GroupLogin(const char *g,const char *p);
+
+   // reset location-related state on Connect/Login/AnonymousLogin
+   virtual void ResetLocationData();
 
    virtual void Open(const char *file,int mode,off_t pos=0);
    void SetFileURL(const char *u) { xfree(file_url); file_url=xstrdup(u); }
@@ -205,6 +214,8 @@ public:
    virtual void	Rename(const char *rfile,const char *to);
    virtual void Mkdir(const char *rfile,bool allpath=false);
    virtual void Chdir(const char *dir,bool verify=true);
+   void OptimizePath(char *path);
+   void SetCwd(const char *dir) { xfree(cwd); cwd=xstrdup(dir); }
    void Remove(const char *rfile)    { Open(rfile,REMOVE); }
    void RemoveDir(const char *dir)  { Open(dir,REMOVE_DIR); }
    void Chmod(const char *file,int m);
@@ -276,9 +287,10 @@ public:
    virtual void CleanupThis();
    void CleanupAll();
       // ^^ close idle connections, etc.
-   virtual ListInfo *MakeListInfo();
+   virtual ListInfo *MakeListInfo(const char *path=0);
    virtual Glob *MakeGlob(const char *pattern);
    virtual DirList *MakeDirList(ArgV *a);
+   virtual FileSet *ParseLongList(const char *buf,int len,int *err=0) { return 0; }
 
    static bool NotSerious(int err);
 
@@ -321,6 +333,7 @@ public:
 	 priority=p;
 	 current->Timeout(0);
       }
+   int GetPriority() const { return priority; }
 
    // not pretty (FIXME)
    int GetRetries() { return retries; }
@@ -355,66 +368,19 @@ public:
    void UseCache(bool y=true) { use_cache=y; }
 };
 
-class Glob : public FileAccessOperation
-{
-protected:
-   char  *pattern;
-   FileSet list;
-   bool	 dirs_only;
-   bool	 files_only;
-   bool	 match_period;
-   bool	 inhibit_tilde;
-   void	 add(const FileInfo *info);
-   void	 add_force(const FileInfo *info);
-   virtual ~Glob();
-public:
-   const char *GetPattern() { return pattern; }
-   FileSet *GetResult() { return &list; }
-   Glob(const char *p);
-   void DirectoriesOnly() { dirs_only=true; }
-   void FilesOnly() { files_only=true; }
-   void NoMatchPeriod() { match_period=false; }
-   void NoInhibitTilde() { inhibit_tilde=false; }
-   void SortByName() { list.SortByName(); }
-
-   static bool HasWildcards(const char *);
-   static void UnquoteWildcards(char *);
-};
-class NoGlob : public Glob
-{
-public:
-   NoGlob(const char *p);
-   const char *Status() { return ""; }
-   int Do();
-};
-
-class GlobURL
-{
-   FileAccess *session;
-   bool reuse;
-   char *url_prefix;
-public:
-   Glob *glob;
-   GlobURL(FileAccess *s,const char *p);
-   ~GlobURL();
-   FileSet *GetResult();
-   bool Done()  { return glob->Done(); }
-   bool Error() { return glob->Error(); }
-   const char *ErrorText() { return glob->ErrorText(); }
-   const char *Status() { return glob->Status(); }
-   void SortByName() { glob->SortByName(); }
-};
-
 #include "FileSet.h"
 
+class PatternSet;
+
 class ListInfo : public FileAccessOperation
 {
 protected:
+   FileAccess *session;
+   char *saved_cwd;
    FileSet *result;
 
-   const char *path;
-   regex_t *rxc_exclude;
-   regex_t *rxc_include;
+   const char *exclude_prefix;
+   PatternSet *exclude;
 
    unsigned need;
    bool follow_symlinks;
@@ -422,10 +388,11 @@ protected:
    virtual ~ListInfo();
 
 public:
-   ListInfo();
+   ListInfo(FileAccess *session,const char *path);
 
-   virtual void SetExclude(const char *p,regex_t *x,regex_t *i);
+   void SetExclude(const char *p,PatternSet *e);
 
+   // caller has to delete the resulting FileSet itself.
    FileSet *GetResult()
       {
 	 FileSet *tmp=result;
@@ -447,12 +414,15 @@ class DirList : public FileAccessOperation
 protected:
    Buffer *buf;
    ArgV *args;
+   bool color;
+
    ~DirList();
 public:
    DirList(ArgV *a)
       {
 	 buf=new Buffer();
 	 args=a;
+	 color=false;
       }
 
    virtual int Do() = 0;
@@ -462,6 +432,7 @@ public:
    bool Eof() { return buf->Eof();  }
    void Get(const char **b,int *size) { buf->Get(b,size); }
    void Skip(int len) { buf->Skip(len); }
+   void UseColor(bool c=true) { color=c; }
 };
 
 
@@ -481,6 +452,8 @@ public:
 
    // start with n==0, then increase n; returns 0 when no more
    static FileAccess *Walk(int *n,const char *proto);
+
+   static void ClearAll();
 };
 
 #endif /* FILEACCESS_H */
