@@ -18,7 +18,7 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: FileCopy.cc,v 1.71 2001/02/03 11:37:35 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.86 2002/06/05 09:29:51 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -72,6 +72,9 @@ int FileCopy::Do()
       state=INITIAL;
       m=MOVED;
    case(INITIAL):
+      if(remove_target_first && !put->FileRemoved())
+	 return m;
+      remove_target_first=false;
       if(put->NeedSizeDateBeforehand())
       {
 	 if(get->GetSize()==NO_SIZE_YET || get->GetDate()==NO_DATE_YET)
@@ -142,6 +145,11 @@ int FileCopy::Do()
       if(put->Broken())
       {
 	 debug((9,_("copy: put is broken\n")));
+	 if(fail_if_broken)
+	 {
+	    SetError(strerror(EPIPE));
+	    return MOVED;
+	 }
 	 goto pre_GET_DONE_WAIT;
       }
       put->Resume();
@@ -182,7 +190,7 @@ int FileCopy::Do()
 	    }
 	    debug((9,_("copy: put rolled back to %lld, seeking get accordingly\n"),
 		     (long long)put->GetRealPos()));
-	    debug((10,_("copy: get position was %lld\n"),
+	    debug((10,"copy: get position was %lld\n",
 		     (long long)get->GetRealPos()));
 	    get->Seek(put->GetRealPos());
 	    return MOVED;
@@ -220,7 +228,7 @@ int FileCopy::Do()
       get->Get(&b,&s);
       if(b==0) // eof
       {
-	 debug((10,_("copy: get hit eof\n")));
+	 debug((10,"copy: get hit eof\n"));
       eof:
 	 put->Resume();
 	 if(line_buffer)
@@ -293,7 +301,7 @@ int FileCopy::Do()
 
       if(get->range_limit!=FILE_END && get->range_limit<=get->GetRealPos())
       {
-	 debug((10,_("copy: get reached range limit\n")));
+	 debug((10,"copy: get reached range limit\n"));
 	 goto eof;
       }
       return m;
@@ -315,7 +323,7 @@ int FileCopy::Do()
 
       if(!put->Done())
 	 return m;
-      debug((10,_("copy: put confirmed store\n")));
+      debug((10,"copy: put confirmed store\n"));
 
    pre_GET_DONE_WAIT:
       get->Empty();
@@ -324,7 +332,6 @@ int FileCopy::Do()
       state=GET_DONE_WAIT;
       m=MOVED;
       end_time=now;
-      end_time_ms=now_ms;
       Delete(put); put=0;
       /* fallthrough */
    case(GET_DONE_WAIT):
@@ -337,7 +344,7 @@ int FileCopy::Do()
       }
       if(!get->Done())
 	 return m;
-      debug((10,_("copy: get is finished - all done\n")));
+      debug((10,"copy: get is finished - all done\n"));
       state=ALL_DONE;
       Delete(get); get=0;
       return MOVED;
@@ -371,12 +378,10 @@ void FileCopy::Init()
    put_buf=0;
    put_eof_pos=0;
    bytes_count=0;
-   start_time=0;
-   start_time_ms=0;
-   end_time=0;
-   end_time_ms=0;
    fail_if_cannot_seek=false;
+   fail_if_broken=true;
    remove_source_later=false;
+   remove_target_first=false;
    line_buffer=0;
    line_buffer_max=0;
 }
@@ -540,7 +545,8 @@ long FileCopy::GetETA(off_t b)
 }
 const char *FileCopy::GetStatus()
 {
-   static char buf[256];
+   static char *buf;
+   xfree(buf); buf=0;
    const char *get_st=0;
    if(get)
       get_st=get->GetStatus();
@@ -548,28 +554,21 @@ const char *FileCopy::GetStatus()
    if(put)
       put_st=put->GetStatus();
    if(get_st && get_st[0] && put_st && put_st[0])
-      sprintf(buf,"[%s->%s]",get_st,put_st);
+      buf=xasprintf("[%s->%s]",get_st,put_st);
    else if(get_st && get_st[0])
-      sprintf(buf,"[%s]",get_st);
+      buf=xasprintf("[%s]",get_st);
    else if(put_st && put_st[0])
-      sprintf(buf,"[%s]",put_st);
+      buf=xasprintf("[%s]",put_st);
    else
       return "";
    return buf;
 }
 
-time_t FileCopy::GetTimeSpent()
+double FileCopy::GetTimeSpent()
 {
-   if(start_time==0 || end_time==0 || end_time<start_time)
+   if(end_time<start_time)
       return 0;
-   return end_time-start_time;
-}
-int FileCopy::GetTimeSpentMilli()
-{
-   if(start_time==0 || end_time==0 || end_time<start_time
-   || (end_time==start_time && end_time_ms<start_time_ms))
-      return 0;
-   return end_time_ms-start_time_ms;
+   return TimeDiff(end_time,start_time);
 }
 
 FgData *FileCopy::GetFgData(bool fg)
@@ -631,10 +630,7 @@ bool FileCopyPeer::Done()
       if(removing)
 	 return false;
       if(mode==PUT)
-      {
-	 if(do_set_date && !date_set)
-	    return false;
-      }
+	 return done;
       return true;
    }
    return false;
@@ -649,15 +645,18 @@ FileCopyPeer::FileCopyPeer(direction m)
    e_size=NO_SIZE;
    date=NO_DATE_YET;
    seek_pos=0;
-   can_seek=true;
-   can_seek0=true;
+   can_seek=false;
+   can_seek0=false;
    date_set=false;
    do_set_date=true;
    ascii=false;
    range_start=0;
    range_limit=FILE_END;
    removing=false;
+   file_removed=false;
    use_cache=true;
+   write_allowed=true;
+   done=false;
    Suspend();  // don't do anything too early
 }
 FileCopyPeer::~FileCopyPeer()
@@ -678,8 +677,10 @@ int FileCopyPeerFA::Do()
       if(res<=0)
       {
 	 removing=false;
+	 file_removed=true;
 	 session->Close();
-	 m=MOVED;
+	 Suspend();
+	 return MOVED;
       }
       else
 	 return m;
@@ -741,6 +742,7 @@ int FileCopyPeerFA::Do()
 	    fxp_eof:
 	       // FIXME: set date for real.
 	       date_set=true;
+	       done=true;
 	       m=MOVED;
 	    }
 	    else if(res==FA::IN_PROGRESS)
@@ -841,7 +843,7 @@ void FileCopyPeerFA::OpenSession()
       if(size!=NO_SIZE && size!=NO_SIZE_YET && seek_pos>=size && !ascii)
       {
       past_eof:
-	 debug((10,_("copy src: seek past eof (seek_pos=%lld, size=%lld)\n"),
+	 debug((10,"copy src: seek past eof (seek_pos=%lld, size=%lld)\n",
 		  (long long)seek_pos,(long long)size));
 	 pos=seek_pos;
 	 eof=true;
@@ -859,30 +861,6 @@ void FileCopyPeerFA::OpenSession()
 	 Save(0);
 	 Allocate(s);
 	 memmove(buffer+buffer_ptr,b,s);
-      #ifndef NATIVE_CRLF
-	 if(ascii)
-	 {
-	    char *buf=buffer+buffer_ptr;
-	    // find where line ends.
-	    char *cr=buf;
-	    for(;;)
-	    {
-	       cr=(char *)memchr(cr,'\r',s-(cr-buf));
-	       if(!cr)
-		  break;
-	       if(cr-buf<s-1 && cr[1]=='\n')
-	       {
-		  memmove(cr,cr+1,s-(cr-buf)-1);
-		  s--;
-		  if(s<=1)
-		     break;
-	       }
-	       else if(cr-buf==s-1)
-		  break;
-	       cr++;
-	    }
-	 }
-      #endif	 // NATIVE_CRLF
 	 in_buffer=s;
 	 pos=seek_pos;
 	 eof=true;
@@ -959,6 +937,8 @@ int FileCopyPeerFA::Get_LL(int len)
 	       SetError(_("Too many redirections"));
 	       return -1;
 	    }
+	    if(FAmode==FA::QUOTE_CMD)
+	       FAmode=FA::RETRIEVE;
 
 	    char *loc=alloca_strdup(loc_c);
 	    session->Close(); // loc_c is no longer valid.
@@ -1112,6 +1092,8 @@ void FileCopyPeerFA::Init()
    try_time=0;
    retries=0;
    redirections=0;
+   can_seek=true;
+   can_seek0=true;
 }
 
 FileCopyPeerFA::FileCopyPeerFA(FileAccess *s,const char *f,int m)
@@ -1192,6 +1174,8 @@ FileCopyPeerFDStream::FileCopyPeerFDStream(FDStream *o,direction m)
 	 seek_base=0;
       }
    }
+   if(stream->usesfd(1))
+      write_allowed=false;
 }
 FileCopyPeerFDStream::~FileCopyPeerFDStream()
 {
@@ -1261,6 +1245,7 @@ int FileCopyPeerFDStream::getfd()
       }
       return -1;
    }
+   stream->clear_status();
    pos=0;
    if(mode==PUT)
       pos+=in_buffer;
@@ -1270,9 +1255,8 @@ int FileCopyPeerFDStream::getfd()
 int FileCopyPeerFDStream::Do()
 {
    int m=STALL;
-   int res;
    if(Done() || Error())
-      return STALL;
+      return m;
    switch(mode)
    {
    case PUT:
@@ -1280,45 +1264,61 @@ int FileCopyPeerFDStream::Do()
       {
 	 if(eof)
 	 {
-	    if(date_set || date==NO_DATE_YET)
-	       return m;
-	    if(date!=NO_DATE && do_set_date)
+	    if(!date_set && date!=NO_DATE && do_set_date)
 	    {
+	       if(date==NO_DATE_YET)
+		  return m;
 	       if(getfd()==-1)
 		  return m;
 	       stream->setmtime(date);
+	       date_set=true;
+	       m=MOVED;
 	    }
-	    date_set=true;
+	    if(stream && delete_stream && !stream->Done())
+	       return m;
+	    done=true;
 	    return MOVED;
 	 }
 	 if(seek_pos==0)
 	    return m;
       }
-      res=Put_LL(buffer+buffer_ptr,in_buffer);
-      if(res>0)
+      if(!write_allowed)
+	 return m;
+      while(in_buffer>0)
       {
-	 in_buffer-=res;
-	 buffer_ptr+=res;
-	 return MOVED;
+	 int res=Put_LL(buffer+buffer_ptr,in_buffer);
+	 if(res>0)
+	 {
+	    in_buffer-=res;
+	    buffer_ptr+=res;
+	    m=MOVED;
+	 }
+	 if(res<0)
+	    return MOVED;
+	 if(res==0)
+	    break;
       }
-      if(res<0)
-	 return MOVED;
       break;
 
    case GET:
       if(eof)
 	 return m;
-      res=Get_LL(GET_BUFSIZE);
-      if(res>0)
+      while(in_buffer<GET_BUFSIZE)
       {
-	 in_buffer+=res;
-	 SaveMaxCheck(0);
-	 return MOVED;
+	 int res=Get_LL(GET_BUFSIZE);
+	 if(res>0)
+	 {
+	    in_buffer+=res;
+	    SaveMaxCheck(0);
+	    m=MOVED;
+	 }
+	 if(res<0)
+	    return MOVED;
+	 if(eof)
+	    return MOVED;
+	 if(res==0)
+	    break;
       }
-      if(res<0)
-	 return MOVED;
-      if(eof)
-	 return MOVED;
       break;
    }
    return m;
@@ -1329,15 +1329,6 @@ bool FileCopyPeerFDStream::IOReady()
    return seek_pos==pos || stream->fd!=-1;
 }
 
-bool FileCopyPeerFDStream::Done()
-{
-   if(!super::Done())
-      return false;
-   if(stream && delete_stream && !stream->Done())
-      return false;
-   return true;
-}
-
 void FileCopyPeerFDStream::Seek(off_t new_pos)
 {
    if(pos==new_pos)
@@ -1423,10 +1414,13 @@ int FileCopyPeerFDStream::Get_LL(int len)
 	 Block(fd,POLLIN);
 	 return 0;
       }
+      if(stream->NonFatalError(errno))
+	 return 0;
       stream->MakeErrorText();
       SetError(stream->error_text);
       return -1;
    }
+   stream->clear_status();
 
 #ifndef NATIVE_CRLF
    if(ascii)
@@ -1508,10 +1502,13 @@ int FileCopyPeerFDStream::Put_LL(const char *buf,int len)
 	 broken=true;
 	 return -1;
       }
+      if(stream->NonFatalError(errno))
+	 return 0;
       stream->MakeErrorText();
       SetError(stream->error_text);
       return -1;
    }
+   stream->clear_status();
    if(res==len)
       res+=skip_cr;
    return res;
@@ -1529,8 +1526,21 @@ void FileCopyPeerFDStream::RemoveFile()
 {
    stream->remove();
    removing=false;   // it is instant.
+   file_removed=true;
+   Suspend();
+   current->Timeout(0);
 }
 
+const char *FileCopyPeerFDStream::GetStatus()
+{
+   return stream->status;
+}
+void FileCopyPeerFDStream::Kill(int sig)
+{
+   stream->Kill(sig);
+}
+
+
 FileCopyPeerFDStream *FileCopyPeerFDStream::NewPut(const char *file,bool cont)
 {
    return new FileCopyPeerFDStream(new FileStream(file,O_WRONLY|O_CREAT
@@ -1554,6 +1564,8 @@ FileCopyPeerString::FileCopyPeerString(const char *s, int len)
    Put(s,len);
    eof=true;
    pos=0;
+   can_seek=true;
+   can_seek0=true;
 }
 FileCopyPeerString::~FileCopyPeerString()
 {
