@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2000 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 2000-2001 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -18,13 +18,13 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: Fish.h,v 1.9 2000/12/22 14:03:56 lav Exp $ */
+/* $Id: Fish.h,v 1.18 2001/12/20 11:14:43 lav Exp $ */
 
 #ifndef FISH_H
 #define FISH_H
 
 #include "NetAccess.h"
-#include "StatusLine.h"
+#include "PtyShell.h"
 
 class Fish : public NetAccess
 {
@@ -32,6 +32,7 @@ class Fish : public NetAccess
    {
       DISCONNECTED,
       CONNECTING,
+      CONNECTING_1,
       CONNECTED,
       FILE_RECV,
       FILE_SEND,
@@ -40,18 +41,20 @@ class Fish : public NetAccess
    };
 
    state_t state;
+   bool received_greeting;
 
    void Init();
 
    int max_send;
    void	 Send(const char *format,...) PRINTF_LIKE(2,3);
    void	 SendMethod();
+   void	 SendArrayInfoRequests();
 
    IOBuffer *send_buf;
    IOBuffer *recv_buf;
+   bool recv_buf_suspended;
 
-   OutputFilter *filter_out;  // used in connecting
-   int pipe_in[2];	      // used in connecting
+   PtyShell *ssh;
 
    void Disconnect();
    int IsConnected()
@@ -74,11 +77,18 @@ class Fish : public NetAccess
       EXPECT_CWD,
       EXPECT_DIR,
       EXPECT_RETR_INFO,
-      EXPECT_RETR
+      EXPECT_RETR,
+      EXPECT_INFO,
+      EXPECT_DEFAULT,
+      EXPECT_STOR_PRELIMINARY,
+      EXPECT_STOR,
+      EXPECT_QUOTE,
+      EXPECT_IGNORE
    };
 
    void PushExpect(expect_t);
    int HandleReplies();
+   void CloseExpectQueue();
    int ReplyLogPriority(int);
 
    expect_t *RespQueue;
@@ -96,15 +106,14 @@ class Fish : public NetAccess
    int	 RespQueueSize() { return RQ_tail-RQ_head; }
    void  EmptyRespQueue() { RQ_head=RQ_tail=0; }
 
-   void GetBetterConnection(int level,int count);
+   void GetBetterConnection(int level);
    void MoveConnectionHere(Fish *o);
 
    char  *line;
    char  *message;
 
    bool	 eof;
-
-   const char *shell_encode(const char *);
+   bool	 encode_file;
 
 public:
    static void ClassInit();
@@ -123,7 +132,7 @@ public:
    int Read(void *,int);
    int Write(const void *,int);
    int StoreStatus();
-   int SendEOT();
+   int Buffered();
 
    void Close();
    const char *CurrentStatus();
@@ -134,6 +143,20 @@ public:
    bool SameLocationAs(FileAccess *fa);
 
    DirList *MakeDirList(ArgV *args);
+   Glob *MakeGlob(const char *pattern);
+   ListInfo *MakeListInfo(const char *p);
+   FileSet *ParseLongList(const char *buf,int len,int *err=0);
+
+   static const char *shell_encode(const char *);
+   void DontEncodeFile() { encode_file=false; }
+
+   bool NeedSizeDateBeforehand() { return true; }
+
+   void Suspend();
+   void Resume();
+
+   void Cleanup();
+   void CleanupThis();
 };
 
 class FishDirList : public DirList
@@ -152,4 +175,15 @@ public:
    void Resume();
 };
 
+class FishListInfo : public GenericParseListInfo
+{
+   FileSet *Parse(const char *buf,int len);
+public:
+   FishListInfo(Fish *session,const char *path)
+      : GenericParseListInfo(session,path)
+      {
+	 can_get_prec_time=false;
+      }
+};
+
 #endif
